/*
This file is part of dynwatch, a win32 system tray applet which
updates automatically the dyndns entry of quake.gr.

Copyright (c) 2005 John Tsiombikas <nuclear@siggraph.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <windows.h>
#include "watch.h"
#include "gui.h"

extern int running;

long CALLBACK win_proc(HWND win, unsigned int msg, unsigned int wparam, long lparam) {
	switch(msg) {
	case WM_CLOSE:
		DestroyWindow(win);
		PostQuitMessage(0);
		return 0;

	case WM_COMMAND:
		if((HWND)lparam == bn_tray) {
			trayfied = !trayfied;
			if(trayfied && !running) {
				info_msg("If you want to have this program operational, you MUST configure first.\nGoing to the tray but keep in mind that I can't do anything at this point without configuration!");
			}
			tray(trayfied);
		}

		if((HWND)lparam == bn_exit) {
			DestroyWindow(win);
			PostQuitMessage(0);
		}

		if((HWND)lparam == bn_config) {
			en_set_text(en_dnshost, dhost);
			en_set_text(en_user, user);
			en_set_text(en_pass, pass);
			en_set_text(en_hosts, hosts);
			ShowWindow(win_cfg, 1);
		}
		return 0;

	case WM_USER:
		if(lparam == WM_LBUTTONUP) {
			trayfied = !trayfied;
			tray(trayfied);
		}
		return 0;

	case WM_TIMER:
		if(need_update()) {
			update_dns();
		}
		return 0;

	default:
		break;
	}

	return DefWindowProc(win, msg, wparam, lparam);
}


int empty_str(const char *str) {
	while(*str) if(isalnum(*str++)) return 0;
	return 1;
}

long CALLBACK cfg_proc(HWND win, unsigned int msg, unsigned int wparam, long lparam) {
	switch(msg) {
	case WM_COMMAND:
		if((HWND)lparam == bn_ok) {
			if(empty_str(en_get_text(en_dnshost)) ||
				empty_str(en_get_text(en_user)) ||
				empty_str(en_get_text(en_pass)) ||
				empty_str(en_get_text(en_hosts))) {
				err_msg("you must fill all the fields of this configuration dialog!");
				return 0;
			}

			strncpy(dhost, en_get_text(en_dnshost), 256);
			strncpy(user, en_get_text(en_user), 256);
			strncpy(pass, en_get_text(en_pass), 256);
			strncpy(hosts, en_get_text(en_hosts), 256);

			save_config();
			running = 1;
			ShowWindow(win_cfg, 0);
		}

		if((HWND)lparam == bn_cancel) {
			ShowWindow(win_cfg, 0);
		}
		return 0;

	default:
		break;
	}

	return DefWindowProc(win, msg, wparam, lparam);
}