/*
This file is part of dynwatch, a win32 system tray applet which
updates automatically the dyndns entry of quake.gr.

Copyright (c) 2005 John Tsiombikas <nuclear@siggraph.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"
#include "events.h"
#include "resource.h"

HWND win_main, bn_exit, bn_tray, bn_config;
HWND win_cfg, lb_dnshost, lb_user, lb_pass, lb_hosts;
HWND en_dnshost, en_user, en_pass, en_hosts;
HWND bn_ok, bn_cancel;
int trayfied;

void create_gui(void) {
	static const int lb_xsz = 130;
	static const int lb_ysz = 20;
	static const int en_xsz = 100;
	static const int en_ysz = 20;
	static const int yinc = 25;
	int ypos = 5;

	reg_win_class("dynwatch", win_proc);
	reg_win_class("dynwatch.config", cfg_proc);

	win_main = create_window("dynwatch 1.0", 330, 80, "dynwatch");
	bn_config = create_button("Configure", 5, 5, 100, 30, win_main);
	bn_tray = create_button("Send to tray", 110, 5, 100, 30, win_main);
	bn_exit = create_button("Exit", 215, 5, 100, 30, win_main);
	ShowWindow(win_main, 1);

	win_cfg = create_window("Configuration", 270, 190, "dynwatch.config");

	lb_dnshost = create_label("dyndns host:", 5, ypos, lb_xsz, lb_ysz, win_cfg);
	en_dnshost = create_entry("", lb_xsz + 10, ypos, en_xsz, en_ysz, win_cfg);
	ypos += yinc;
	lb_user = create_label("user name:", 5, ypos, lb_xsz, lb_ysz, win_cfg);
	en_user = create_entry("", lb_xsz + 10, ypos, en_xsz, en_ysz, win_cfg);
	ypos += yinc;
	lb_pass = create_label("password:", 5, ypos, lb_xsz, lb_ysz, win_cfg);
	en_pass = create_entry("", lb_xsz + 10, ypos, en_xsz, en_ysz, win_cfg);
	ypos += yinc;
	lb_hosts = create_label("hosts to update:", 5, ypos, lb_xsz, lb_ysz, win_cfg);
	en_hosts = create_entry("", lb_xsz + 10, ypos, en_xsz, en_ysz, win_cfg);

	ypos += 35;
	bn_ok = create_button("OK", 5, ypos, 100, 25, win_cfg);
	bn_cancel = create_button("Cancel", 140, ypos, 100, 25, win_cfg);
}

void tray(int trayfy) {
	NOTIFYICONDATA nicon;
	memset(&nicon, 0, sizeof nicon);
	nicon.cbSize = sizeof nicon;
	nicon.hWnd = win_main;
	nicon.uID = 0;
	nicon.uCallbackMessage = WM_USER;
	nicon.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
	nicon.hIcon = LoadIcon(GetModuleHandle(0), MAKEINTRESOURCE(ICON_DYNWATCH));
	strcpy(nicon.szTip, "Click to open the program window");

	if(trayfy) {
		ShowWindow(win_main, 0);
        Shell_NotifyIcon(NIM_ADD, &nicon);
	} else {
		Shell_NotifyIcon(NIM_DELETE, &nicon);
		ShowWindow(win_main, 1);
	}
}

HWND create_window(const char *title, int xsz, int ysz, const char *class_name) {
	int x = (GetSystemMetrics(SM_CXSCREEN) - xsz) / 2;
	int y = (GetSystemMetrics(SM_CYSCREEN) - ysz) / 2;
	unsigned long style = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU;
	return CreateWindow(class_name, title, style, x, y, xsz, ysz, 0, 0, GetModuleHandle(0), 0);
}

HWND create_button(const char *title, int x, int y, int xsz, int ysz, HWND parent) {
	unsigned long style = BS_PUSHBUTTON | WS_CHILD | WS_VISIBLE | WS_TABSTOP;
	return CreateWindow("BUTTON", title, style, x, y, xsz, ysz, parent, 0, GetModuleHandle(0), 0);
}

HWND create_label(const char *text, int x, int y, int xsz, int ysz, HWND parent) {
	unsigned long style = SS_RIGHT | WS_CHILD | WS_VISIBLE;
	return CreateWindow("STATIC", text, style, x, y, xsz, ysz, parent, 0, GetModuleHandle(0), 0);
}

HWND create_entry(const char *text, int x, int y, int xsz, int ysz, HWND parent) {
	unsigned long style = ES_AUTOHSCROLL | WS_CHILD | WS_BORDER | WS_TABSTOP | WS_VISIBLE;
	return CreateWindow("EDIT", text, style, x, y, xsz, ysz, parent, 0, GetModuleHandle(0), 0);
}

void en_set_text(HWND entry, const char *text) {
	SendMessage(entry, WM_SETTEXT, 0, (long)text);
}

const char *en_get_text(HWND entry) {
	static char buf[512];
	SendMessage(entry, WM_GETTEXT, 512, (long)buf);
	return buf;
}

void reg_win_class(const char *class_name, msg_handler_t handler) {
	WNDCLASSEX wc;
	HINSTANCE pid = GetModuleHandle(0);

	memset(&wc, 0, sizeof wc);
	wc.cbSize = sizeof wc;
	wc.hbrBackground = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
	wc.hCursor = LoadCursor(pid, MAKEINTRESOURCE(IDC_ARROW));
	wc.hIcon = wc.hIconSm = LoadIcon(pid, MAKEINTRESOURCE(ICON_DYNWATCH));
	wc.hInstance = pid;
	wc.lpfnWndProc = handler;
	wc.lpszClassName = class_name;
	wc.style = CS_HREDRAW | CS_VREDRAW;
	RegisterClassEx(&wc);
}